<?php
// $Id: feedapi_eparser.plugins.inc,v 1.13 2009/08/17 22:09:18 neclimdul Exp $
/**
 * @file
 * Functions related to our plugin functionality.
 *
 */

/**
 * CTools plugin information callback for payment api method plugins.
 *
 * @return
 * An array of information about the plugin api.
 */
function eparse_ctools_plugin_type_plugins() {
  return array(
    // This is cacheable.
    'cache' => TRUE,
    // Provide additional default values for definition arrays.
    'defaults' => array(
      'name' => '',
      'description' => '',
      'parent' => 'parent',
      'system' => FALSE,
    ),
  );
}

function eparse_ctools_plugin_namespace_plugins() {
  return array(
    // This is cacheable.
    'cache' => TRUE,
    // Provide additional default values for definition arrays.
    'defaults' => array(
      'name' => '',
      'description' => '',
      'parent' => 'parent',
      'system' => FALSE,
    ),
  );
}

function eparse_ctools_plugin_download_plugins() {
  return array(
    // This is cacheable.
    'cache' => TRUE,
    // Provide additional default values for definition arrays.
    'defaults' => array(
      'name' => '',
      'description' => '',
    ),
  );
}

function feedapi_eparser_fetch_http_request($feed) {
  $url = $feed->url;
  if (valid_url($url, TRUE)) {
    // Handle password protected feeds.
    $url_parts = parse_url($url);
    $password = $username = NULL;
    if (!empty($url_parts['user'])) {
      $password = $url_parts['pass'];
      $username = $url_parts['user'];
    }
  }

  $headers = array();
  $result = drupal_http_request($url, $headers);
  return empty($result->data) ? FALSE : $result->data;
}

/**
 * Implementation of hook_eparse_type_plugins().
 */
function feedapi_eparser_eparse_type_plugins() {
  $base_path = drupal_get_path('module', 'feedapi_eparser') . '/plugins';
  return array(
    'rss20' => array(
      'name' => 'RSS 2.0',
      'handler' => 'FeedapiEparseTypeRSS20',
      'default namespaces' => array('rss20'),
      'path' => $base_path,
    ),
    'atom' => array(
      'name' => 'Atom',
      'handler' => 'FeedapiEparseTypeAtom10',
      'default namespaces' => array('atom10'),
      'path' => $base_path,
    ),
  );
}

/**
 * Implementation of hook_eparse_download_plugins().
 */
function feedapi_eparser_eparse_download_plugins() {
  return array(
    'drupal_http_request' => array(
      'name' => 'Drupal HTTP Request',
      'description' => 'Uses Drupal\'s Http Request function to fetch feeds.',
      'fetch' => 'feedapi_eparser_fetch_http_request',
    ),
  );
}

/**
 * Implementation of hook_eparse_namespace_plugins().
 */
function feedapi_eparser_eparse_namespace_plugins() {
  $base_path = drupal_get_path('module', 'feedapi_eparser') . '/plugins';
  return array(
    'itunes' => array(
      'url' => 'http://www.itunes.com/dtds/podcast-1.0.dtd',
      'name' => 'iTunes',
      'description' => 'iTunes podcast feed information.',
      'handler' => 'FeedapiEparseNamespaceItunes',
      'path' => $base_path,
    ),
    'feedburner' => array(
      'url' => 'http://rssnamespace.org/feedburner/ext/1.0',
      'name' => 'Feedburner',
      'description' => 'Additional information added by feedburner with information like original media urls.',
      'handler' => 'FeedapiEparseNamespaceFeedburner',
      'path' => $base_path,
    ),
    'rss10_content' => array(
      'url' => 'http://web.resource.org/rss/1.0/modules/content/',
      'name' => 'RSS 1.0 - Content',
      'description' => "RSS Content namespace contains extra items and most importantly the encoded field.",
      'handler' => 'FeedapiEparseNamespaceRSS10Content',
      'path' => $base_path,
    ),
    'rss20' => array(
      'url' => 'http://backend.userland.com/RSS2',
      'name' => 'RSS 2.0',
      'description' => "Seldom used RSS 2.0 namespace. Should only really be explicitly added if RSS 2.0 data is embeded in another feed type and you really know what you're doing.",
      'handler' => 'FeedapiEparseNamespaceRSS20',
      'path' => $base_path,
    ),
    'atom10' => array(
      'url' => 'http://www.ietf.org/rfc/rfc4287.txt',
      'name' => 'Atom 1.0',
      'description' => "....",
      'handler' => 'FeedapiEparseNamespaceAtom10',
      'path' => $base_path,
    ),
  );
}

/**
 * Build a list of namespace plugins.
 *
 * @see feedapi_eparser_get_namespace_plugins().
 * @return
 * An array of namespace plugins.
 */
function _feedapi_eparser_get_namespace_plugins() {
  ctools_include('plugins');
  return ctools_get_plugins('eparse', 'namespace_plugins');
}

/**
 * Build a list of download plugins.
 *
 * @see feedapi_eparser_get_download_plugins().
 * @return
 * An array of namespace plugins.
 */
function _feedapi_eparser_get_download_plugins() {
  ctools_include('plugins');
  return ctools_get_plugins('eparse', 'download_plugins');
}

/**
 * Build a list of feed_type plugins.
 *
 * @see feedapi_eparser_get_feed_type_plugins().
 * @return unknown_type
 */
function _feedapi_eparser_get_feed_type_plugins() {
  ctools_include('plugins');
  return ctools_get_plugins('eparse', 'type_plugins');
}

function feedapi_eparser_get_plugin_data($name, $type) {
  static $plugins = array();

  if (!isset($plugins[$type])) {
    switch ($type) {
      case 'feed_type':
        $plugins[$type] = _feedapi_eparser_get_feed_type_plugins();
        break;
      case 'download':
        $plugins[$type] = _feedapi_eparser_get_download_plugins();
        break;
      case 'namespaces':
        $plugins[$type] = _feedapi_eparser_get_namespace_plugins();
        break;
      default:
        return FALSE;
    }
  }

  return $plugins[$type][$name];
}

function _feedapi_eparser_load_type_plugin($feed) {
  if (!module_invoke('ctools', 'api_version', '1.1.1')) {
    return;
  }
  ctools_include('plugins');
  $class = ctools_plugin_load_class('eparse', 'type_plugins', $feed->eparse->type, 'handler');
  if ($class) {
    return new $class;
  }
  return false;
}

interface FeedapiEparserPluginInterface {
  function init(&$feed);
  function parseGlobal();
  function parseItem(&$xml_item, &$item);
}

abstract class FeedapiEparserPluginBase {

  /**
   * Store a reference to the feed object.
   */
  protected $feed;

  /**
   *
   */
  function init(&$feed) {
    $this->feed = &$feed;
  }

  /**
   * Parse a date comes from a feed.
   *
   * @param $date_string
   *  The date string in various formats.
   * @return
   *  The timestamp of the string or the current time if can't be parsed
   */
  function parseDate($date_str) {
    static $time;
    if (!isset($time)) {
      $time = time();
    }

    $parsed_date = strtotime($date_str);
    if ($parsed_date === FALSE || $parsed_date == -1) {
      $parsed_date = $this->parse_w3c_dtf($date_str);
    }
    return $parsed_date === FALSE ? $time-- : $parsed_date;
  }

  /**
   * Parse the W3C date/time format, a subset of ISO 8601.
   *
   * PHP date parsing functions do not handle this format.
   * See http://www.w3.org/TR/NOTE-datetime for more information.
   * Originally from MagpieRSS (http://magpierss.sourceforge.net/).
   *
   * @param $date_str
   *   A string with a potentially W3C DTF date.
   * @return
   *   A timestamp if parsed successfully or FALSE if not.
   */
  function parseW3CDTF($date_str) {
    if (preg_match('/(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2})(:(\d{2}))?(?:([-+])(\d{2}):?(\d{2})|(Z))?/', $date_str, $match)) {
      list($year, $month, $day, $hours, $minutes, $seconds) = array($match[1], $match[2], $match[3], $match[4], $match[5], $match[6]);
      // Calculate the epoch for current date assuming GMT.
      $epoch = gmmktime($hours, $minutes, $seconds, $month, $day, $year);
      if ($match[10] != 'Z') { // Z is zulu time, aka GMT
        list($tz_mod, $tz_hour, $tz_min) = array($match[8], $match[9], $match[10]);
        // Zero out the variables.
        if (!$tz_hour) {
          $tz_hour = 0;
        }
        if (!$tz_min) {
          $tz_min = 0;
        }
        $offset_secs = (($tz_hour * 60) + $tz_min) * 60;
        // Is timezone ahead of GMT?  If yes, subtract offset.
        if ($tz_mod == '+') {
          $offset_secs *= -1;
        }
        $epoch += $offset_secs;
      }
      return $epoch;
    }
    else {
      return FALSE;
    }
  }

  /**
   * Convert strings from formats like simpleXML objects to strings.
   *
   * Helps by patching large groups of conversions. Drupal array love...
   *
   * TODO - Allow some way to force certain keys to exist.
   * TODO - Allow some way to force keys to be multiple values. SimpleXML sucks at these sort of things.
   *
   * @param $map
   * An array of elements in the context.
   * @param $context
   * A simpleXML object that is our current context
   * @return
   * An object containing the converted strings.
   */
  function convertStrings($map, $context) {
    $return = new stdClass;
    foreach ($map as $key => $item) {
      if (is_array($item) && isset($context->$key)) {
        $return->$key = $this->convertStrings($item, $context->$key);
      }
      elseif (!is_array($item) && isset($context->$item)) {
        $return->$item = (string) $context->$item;
      }
    }
    return (object) $return;
  }

  /**
   * Make sure we have a title.
   *
   * We always need a title in FeedAPI but not all feeds actually have one. This helps by creating one out
   * of the first 3 words of the body.
   * @param $title
   * The title, possibly empty.
   * @param $body
   * The body of the content.
   */
  function getItemTitle($title, $body) {
    if (empty($title) && !empty($body)) {
      // Explode to words and use the first 3 words.
      $words = preg_split("/[\s,]+/", $body);
      $title = $words[0] . ' ' . $words[1] . ' ' . $words[2];
    }
    return $title;
  }
}

abstract class FeedapiEparserNamespacePlugin extends FeedapiEparserPluginBase {
  protected $parsed_source;

  function init(&$feed) {
    parent::init($feed);
    // DEPRECATED - We should just use the parsed_source on the feed directly now.
    $this->parsed_source = &$this->feed->parsed_source;
  }

  abstract function parseGlobal($global_context);

  abstract function parseItem(&$source_item, &$destination_item);
}

/**
 * Basic eparser type plugin.
 */
abstract class FeedapiEparserTypePlugin extends FeedapiEparserPluginBase {
  protected $namespace_plugins;

  function init(&$feed) {
    parent::init($feed);

    // Initialize parsed source variable.
    $feed->parsed_source = new stdClass();

    // Store a reference to the type plugin so namespace plugins can interact with it.
    $feed->type_plugin = &$this;

    // If we didn't get a valid feed, fail initializing the plugin.
    if ($this->getGlobalContext() === FALSE) {
      return FALSE;
    }

    // Find default namespace plugins and attach them.
    $def = feedapi_eparser_get_plugin_data($feed->eparse->type, 'feed_type');
    if (!empty($def['default namespaces'])) {
      foreach ($def['default namespaces'] as $namespace) {
        $this->ensureNamespace($namespace);
      }
    }

    // Attach any explicitly added namespace plugins.
    foreach ($feed->eparse->namespaces as $namespace) {
      $this->ensureNamespace($namespace);
    }

    // Intialize all the namespace plugins we've attached.
    foreach ($this->namespace_plugins as $namespace => $plugin) {
      $this->namespace_plugins[$namespace]->init($feed);
    }
  }

  function parse() {
    // The global context is a bit of a hack but it should work for now...
    $global_context = $this->getGlobalContext();

    foreach ($this->namespace_plugins as $plugin) {
      $plugin->parseGlobal($global_context);
    }
    $this->parseGlobal();

    $this->feed->parsed_source->items = array();
    foreach ($this->getItems() as $item) {
      $destination_item = new stdClass();
      foreach ($this->namespace_plugins as $plugin) {
        $plugin->parseItem($item, $destination_item);
      }

      $this->parseItem($item, $destination_item);

      $this->feed->parsed_source->items[] = $destination_item;
    }
  }

  /**
   * Ensure we have a copy of the given namespace in our namespace plugins.
   *
   * @param $namespace
   * @return unknown_type
   */
  function ensureNamespace($namespace) {
    if (!isset($this->namespace_plugins[$namespace])) {
      if (!module_invoke('ctools', 'api_version', '1.1.1')) {
        return;
      }
      ctools_include('plugins');
      $class = ctools_plugin_load_class('eparse', 'namespace_plugins', $namespace, 'handler');
      if ($class) {
        $this->namespace_plugins[$namespace] = new $class;
      }
      else {
        $this->namespace_plugins[$namespace] = false;
      }
    }
  }

  function getParsedSource() {
    return $this->feed->parsed_source;
  }

  /**
   * Return the global context used for namespace plugins to find their global feed information.
   *
   * @return
   * An array.
   */
  abstract function getGlobalContext();

  /**
   *
   * @return unknown_type
   */
  abstract function getItems();

  /**
   * Fetch string data from some location stored on the feed object.
   *
   * @param $feed
   * The feed to download.
   * @return
   * String.
   */
  protected function download($feed) {
    ctools_include('plugins');
    $download = ctools_plugin_load_function('eparse', 'download_plugins', $feed->eparse->download, 'fetch');
    if ($download) {
      return $download($feed);
    }
    else {
      echo "Fail: $download <br/>";
    }
  }
}

/**
 * Basic XML Type Plugin.
 */
abstract class FeedapiEparserTypePluginXML extends FeedapiEparserTypePlugin {

  protected $xml;

  /**
   * Return the global context used for namespace plugins to find their global feed information.
   *
   * @return
   * SimpleXML object.
   */
  function getGlobalContext() {
    // Many feeds will be using the root. The way simpleXML works this will
    // return the root tag as the context.
    print_r($this->feed);
    return $this->getXML();
  }

  /**
   * Get a SimpleXML object containing the data for a feed.
   *
   * @param $feed
   * The feed to be looked up.
   * @return
   * SimpleXML object setup with the content of the feed.
   */
  protected function getXML() {
    if (!isset($this->xml)) {
      $downloaded_string = $this->download($this->feed);

      if (!defined('LIBXML_VERSION') || (version_compare(phpversion(), '5.1.0', '<'))) {
        @$this->xml = simplexml_load_string($downloaded_string, NULL);
      }
      else {
        @$this->xml = simplexml_load_string($downloaded_string, NULL, LIBXML_NOERROR | LIBXML_NOWARNING);
      }
    }

    return $this->xml;
  }
}
